@@ -20,7 +20,7 @@ class UBXMessage():
     UBX Message Class.
     '''
 
-    def __init__(self, ubx_class, ubx_id, payload=b'', mode=ubt.GET):
+    def __init__(self, ubx_class, ubx_id, payload=None, mode=ubt.GET):
         '''
         Constructor.
 
@@ -28,29 +28,25 @@ class UBXMessage():
         '''
 
         self._header = ubt.UBX_HDR
+        self._mode = mode
         if isinstance(ubx_class, str) and isinstance(ubx_id, str):  # e.g. 'CFG, CFG-PRT'
             (self._ubx_class, self._ubx_id) = self.ubx_str2bytes(ubx_class, ubx_id)
         else:  # e.g. b'\x06', b'\x01'
             self._ubx_class = ubx_class
             self._ubx_id = ubx_id
-        if isinstance(payload, str):
-            payload = bytes(payload, 'utf-8')
-        self._length = self.len2bytes(len(payload))
-        self._checksum = self.calc_checksum(self._ubx_class + self._ubx_id +
+        if payload is None:
+            self._length = self.len2bytes(0)
+            self._checksum = self.calc_checksum(self._ubx_class + self._ubx_id +
+                                                self._length)
+        else:
+            if isinstance(payload, str):
+                payload = bytes(payload, 'utf-8')
+            self._length = self.len2bytes(len(payload))
+            self._checksum = self.calc_checksum(self._ubx_class + self._ubx_id +
                                             self._length + payload)
-        self._mode = mode
-
-        if payload:
-            self.payload = payload
+        self.payload = payload
 
     def __str__(self) -> str:
-        '''
-        String representation.
-        '''
-
-        return repr(self)
-
-    def __repr__(self) -> str:
         '''
         Human readable representation.
         '''
@@ -58,7 +54,10 @@ class UBXMessage():
         clsid = None
 
         umsg_name = self.identity
-        rep = f"<UBX({umsg_name}, "
+        if self.payload is None:
+            return f"<UBX({umsg_name})>"
+
+        stg = f"<UBX({umsg_name}, "
         for i, att in enumerate(self.__dict__):
             if att[0] != '_':  # only show public attributes
                 val = self.__dict__[att]
@@ -81,17 +80,30 @@ class UBXMessage():
                             val = ubt.UBX_CONFIG_MESSAGES[clsid + val]
                 except KeyError:
                     pass  # ignore any dictionary lookup errors and just show original binary value
-                rep += att + '=' + str(val)
+                stg += att + '=' + str(val)
                 if i < len(self.__dict__) - 1:
-                    rep += ", "
-        rep += ")>"
-        return rep
+                    stg += ", "
+        stg += ")>"
+
+        return stg
+
+    def __repr__(self) -> str:
+        '''
+        Machine readable representation.
+        '''
+
+        if self._payload is None:
+            return f"'UBXMessage({self._ubx_class}, {self._ubx_id})'"
+        return f"'UBXMessage({self._ubx_class}, {self._ubx_id}, {self._payload})'"
 
     def serialize(self) -> bytes:
         '''
         Return message content as byte array suitable for writing to a stream.
         '''
 
+        if self._payload is None:
+            return (ubt.UBX_HDR + self._ubx_class + self._ubx_id + self._length
+                    +self._checksum)
         return (ubt.UBX_HDR + self._ubx_class + self._ubx_id + self._length
                 +self._payload + self._checksum)
 
@@ -109,10 +121,17 @@ class UBXMessage():
         clsid = message[2:3]
         msgid = message[3:4]
         lenb = message[4:6]
-        payload = message[6:lenm - 2]
-        leni = len(payload)
+        if lenb == b'\x00\x00':
+            payload = None
+            leni = 0
+        else:
+            payload = message[6:lenm - 2]
+            leni = len(payload)
         ckm = message[lenm - 2:lenm]
-        ckv = UBXMessage.calc_checksum(clsid + msgid + lenb + payload)
+        if payload is not None:
+            ckv = UBXMessage.calc_checksum(clsid + msgid + lenb + payload)
+        else:
+            ckv = UBXMessage.calc_checksum(clsid + msgid + lenb)
         if validate:
             if hdr != ubt.UBX_HDR:
                 raise ube.UBXParseError(f"Invalid message header {hdr} - should be {ubt.UBX_HDR}")
@@ -243,7 +262,11 @@ class UBXMessage():
         '''
 
         self._payload = payload
-        self._length = self.len2bytes(len(self._payload))
+        lng = 0 if payload is None else len(payload)
+        self._length = self.len2bytes(lng)
+        if payload is None:
+            return
+
         offset = 0
         self._index = 0
         try:
@@ -278,9 +301,14 @@ class UBXMessage():
         # print(f" _PAYLOAD_ATTR - identity={self.identity}, key = {key}")
         att = pdict[key]  # get attribute type
         if isinstance(att, dict):  # attribute is a dict i.e. a nested repeating group
+            # no 'numCh' attribute for these message types so need to deduce
+            if self.identity in ('AID-ALM', 'CFG-RINV', 'MON-VER', 'RXM-ALM'):
+                rng = self._get_repeats(att, payload, offset)
+            elif self.identity in ('ESF-MEAS', 'RXM-EPH'):
+                rng = self._get_optionals(att, payload, offset)
             # get preceding attribute containing number of items in this repeating group
             # assumed to be 'numCh' unless otherwise specified in UBX_PAYLOADS
-            if self.identity == 'AID-ALPSRV':
+            elif self.identity == 'AID-ALPSRV':
                 rng = self.dataSize
 #             elif self.identity == 'AN-OTHER'
 #                 rng = self.whatever # whatever name is given to the attribute
@@ -313,3 +341,29 @@ class UBXMessage():
             offset += atts
 
         return (offset, att)
+
+    def _get_repeats(self, att, payload : bytes, offset: int) -> int:
+        '''
+        Returns number of items in repeating group
+        where this isn't specified by a 'numCh' attribute
+        
+        NB: this assumes the indeterminate repeating group is 
+        always at the end of the payload
+        '''
+
+        # get length of remaining payload
+        plen = len(payload) - offset
+        # calculate length of each item in group
+        lng = 0
+        for _, val in att.items():
+            lng += int(val[1:3])
+        # deduce number of repeating items in remaining payload
+        return int(plen / lng)
+
+    def _get_optionals(self, att, payload : bytes, offset: int) -> int:
+        '''
+        Returns number of items in optional repeating groups
+        '''
+
+        # TODO - not yet implemented
+        return 0
